﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;

/* --- © 2022 JaxZoa <JaxZoa@gmail.com> --- */

namespace JaxPlugins
{

    public class Step
    {
        protected StepCycle cycle;

        protected float duration;
        protected float delay;
        protected bool right;

        protected float startTime;

        protected Vector3 startPos;
        protected Vector3 tarPos;

        protected Quaternion startRot;
        protected Quaternion tarRot;

        protected bool isActive = false;

        protected float effectiveDuration = 1;
        protected float stepHeightDelta = 0;

        protected bool secondaryStep = false;

        protected AnimationCurve animCurve;

        Keyframe _keyFrame;

        public Step(StepCycle _cycle, float _duration, float _delay, bool _right, bool _secondaryStep = false)
        {
            _keyFrame = new Keyframe();

            cycle = _cycle;

            duration = _duration;
            delay = _delay;
            right = _right;

            secondaryStep = _secondaryStep;

            animCurve = new AnimationCurve();

            animCurve.AddKey(0f, 0f);
            animCurve.AddKey(0.5f, 1f);
            animCurve.AddKey(1f, 0f);

            animCurve.keys[0].weightedMode = WeightedMode.Both;
            animCurve.keys[0].inWeight = 0.5f;
            animCurve.keys[0].outWeight = 0.5f;

            animCurve.keys[1].weightedMode = WeightedMode.Both;
            animCurve.keys[1].inWeight = 0.5f;
            animCurve.keys[1].outWeight = 0.5f;

            animCurve.keys[2].weightedMode = WeightedMode.Both;
            animCurve.keys[2].inWeight = 0.5f;
            animCurve.keys[2].outWeight = 0.5f;
        }

        public void Init(Vector3 _startPos, Quaternion _startRot)
        {
            tarRot = _startRot;
            tarPos = _startPos;
        }

        public void StartStep(Vector3 _tarPos, Quaternion _tarRot)
        {
            isActive = true;

            stepHeightDelta =  _tarPos.y - tarPos.y;

            startPos = tarPos;
            startRot = tarRot;

            tarPos = _tarPos;
            tarRot = _tarRot;

            startTime = Time.time;

            if (cycle.StepInPlace)
                effectiveDuration = duration + Mathf.Clamp(UnityEngine.Random.Range(cycle.StepDurationRandom_Min, cycle.StepDurationRandom_Max), 0f, 5f);
            else
            {
                effectiveDuration = Mathf.Lerp(duration * 0.5f, duration, cycle.manager.MovementSpeedDirectional);
                effectiveDuration = Mathf.Lerp(effectiveDuration * 0.75f, effectiveDuration, Mathf.Clamp01(StepLength));

                effectiveDuration += Mathf.Clamp(UnityEngine.Random.Range(cycle.StepDurationRandom_Min, cycle.StepDurationRandom_Max), 0f, 5f);
            }
        }

        public void ResetStep()
        {
            isActive = false;
        }

        public void SetStepHeight(float _height)
        {
            startPos.y = _height;
            tarPos.y = _height;
        }

        public float StepTime
        {
            get { return Mathf.Clamp01((Time.time - startTime) / effectiveDuration); }
        }

        public float StepLength
        {
            get { return (startPos - tarPos).magnitude; }
        }

        public float GetStepStrength(float _midPoint, bool applyFilters = true)
        {
            _keyFrame.value = 1;
            _keyFrame.time = _midPoint;
            _keyFrame.inWeight = 0.5f;
            _keyFrame.outWeight = 0.5f;

            animCurve.MoveKey(1, _keyFrame);

            if (!applyFilters || cycle.StepInPlace)
                return cycle.StepInPlaceAmplitude * animCurve.Evaluate(StepTime);
            else
                return Mathf.Lerp(0, animCurve.Evaluate(StepTime), Mathf.Clamp01(StepLength));

            /* float _str = Mathf.Pow((4 * StepTime) * (1 - StepTime), cycle.Sharpness);

            if (applyFilters)
            {
                if (cycle.StepInPlace)
                    _str *= cycle.StepInPlaceAmplitude;
                else
                    _str = Mathf.Lerp(_str * 0.15f, _str, Mathf.Clamp01(StepLength));
            }

            return _str; */
        }

        public Quaternion TargetRot
        {
            get { return Quaternion.Lerp(startRot, tarRot, StepTime); }
        }

        public Vector3 TargetPos
        {
            get { return Vector3.Lerp(startPos, tarPos, StepTime); }
        }

        public bool IsActive
        {
            get { return isActive; }
        }

        public bool PlayedThisCycle
        {
            get { return Mathf.Clamp01((Time.time - startTime) / (effectiveDuration + delay)) >= 1; }
        }

        public bool IsIdle
        {
            get { return tarPos == startPos && tarRot == startRot; }
        }

        public bool SecondaryStep
        {
            get { return secondaryStep; }
        }

        public float StepHeightDelta
        {
            get { return stepHeightDelta; }
        }

        public float Duration
        {
            get { return duration; }

            set { duration = value; }
        }

        public float Delay
        {
            get { return delay; }

            set { delay = value; }
        }

        public bool Right
        {
            get { return right; }
        }
    }

}